# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .mcp_server.client import AsyncMcpServerClient, McpServerClient
from .raw_client import AsyncRawToolsClient, RawToolsClient
from .types.cohort_response import CohortResponse
from .types.lang2fhir_and_create_multi_response import Lang2FhirAndCreateMultiResponse
from .types.lang2fhir_and_create_request_resource import Lang2FhirAndCreateRequestResource
from .types.lang2fhir_and_create_response import Lang2FhirAndCreateResponse
from .types.lang2fhir_and_search_response import Lang2FhirAndSearchResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ToolsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawToolsClient(client_wrapper=client_wrapper)
        self.mcp_server = McpServerClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawToolsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawToolsClient
        """
        return self._raw_client

    def create_fhir_resource(
        self,
        *,
        resource: Lang2FhirAndCreateRequestResource,
        text: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        provider: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Lang2FhirAndCreateResponse:
        """
        Converts natural language to FHIR resource and optionally stores it in a FHIR server

        Parameters
        ----------
        resource : Lang2FhirAndCreateRequestResource
            Type of FHIR resource to create. Use 'auto' for automatic resource type detection, or specify a supported US Core profile.

        text : str
            Natural language text to convert to FHIR resource

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        provider : typing.Optional[str]
            FHIR provider ID - must be a valid UUID from existing FHIR providers. also supports provider by name (e.g. medplum)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Lang2FhirAndCreateResponse
            Successfully created FHIR resource

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.tools.create_fhir_resource(
            phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
            phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
            resource="auto",
            text="Patient John Doe has severe asthma with acute exacerbation",
        )
        """
        _response = self._raw_client.create_fhir_resource(
            resource=resource,
            text=text,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            provider=provider,
            request_options=request_options,
        )
        return _response.data

    def create_fhir_resources_multi(
        self,
        *,
        text: str,
        provider: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Lang2FhirAndCreateMultiResponse:
        """
        Extracts multiple FHIR resources from natural language text and stores them in a FHIR server.
        Automatically detects Patient, Condition, MedicationRequest, Observation, and other resource types.
        Resources are linked with proper references and submitted as a transaction bundle.
        For FHIR servers that don't auto-resolve urn:uuid references, this endpoint will automatically
        resolve them via PUT requests after the initial bundle creation.

        Parameters
        ----------
        text : str
            Natural language text containing multiple clinical concepts to extract

        provider : str
            FHIR provider ID or name

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        version : typing.Optional[str]
            FHIR version to use

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Lang2FhirAndCreateMultiResponse
            Successfully created FHIR resources

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.tools.create_fhir_resources_multi(
            phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
            phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
            text="John Smith, 45-year-old male, diagnosed with Type 2 Diabetes. Prescribed Metformin 500mg twice daily.",
            provider="medplum",
        )
        """
        _response = self._raw_client.create_fhir_resources_multi(
            text=text,
            provider=provider,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            version=version,
            request_options=request_options,
        )
        return _response.data

    def search_fhir_resources(
        self,
        *,
        text: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        patient_id: typing.Optional[str] = OMIT,
        practitioner_id: typing.Optional[str] = OMIT,
        count: typing.Optional[int] = OMIT,
        provider: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Lang2FhirAndSearchResponse:
        """
        Converts natural language to FHIR search parameters and executes search in FHIR server

        Parameters
        ----------
        text : str
            Natural language text to convert to FHIR search parameters

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        patient_id : typing.Optional[str]
            Patient ID to filter results

        practitioner_id : typing.Optional[str]
            Practitioner ID to filter results

        count : typing.Optional[int]
            Maximum number of results to return

        provider : typing.Optional[str]
            FHIR provider ID - must be a valid UUID from existing FHIR providers. also supports provider by name (e.g. medplum)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Lang2FhirAndSearchResponse
            Successfully generated search and retrieved results

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.tools.search_fhir_resources(
            phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
            phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
            text="Find all appointments for patient John Doe next week",
        )
        """
        _response = self._raw_client.search_fhir_resources(
            text=text,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            patient_id=patient_id,
            practitioner_id=practitioner_id,
            count=count,
            provider=provider,
            request_options=request_options,
        )
        return _response.data

    def analyze_cohort(
        self,
        *,
        text: str,
        provider: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CohortResponse:
        """
        Uses LLM to extract search concepts from natural language and builds patient cohorts with inclusion/exclusion criteria

        Parameters
        ----------
        text : str
            Natural language text describing the patient cohort criteria

        provider : str
            FHIR provider ID - must be a valid UUID from existing FHIR providers. also supports provider by name (e.g. medplum)

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CohortResponse
            Successfully analyzed cohort and retrieved patient list

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.tools.analyze_cohort(
            phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
            phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
            text="female patients over 20 with diabetes but not hypertension",
            provider="550e8400-e29b-41d4-a716-446655440000",
        )
        """
        _response = self._raw_client.analyze_cohort(
            text=text,
            provider=provider,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            request_options=request_options,
        )
        return _response.data


class AsyncToolsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawToolsClient(client_wrapper=client_wrapper)
        self.mcp_server = AsyncMcpServerClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawToolsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawToolsClient
        """
        return self._raw_client

    async def create_fhir_resource(
        self,
        *,
        resource: Lang2FhirAndCreateRequestResource,
        text: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        provider: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Lang2FhirAndCreateResponse:
        """
        Converts natural language to FHIR resource and optionally stores it in a FHIR server

        Parameters
        ----------
        resource : Lang2FhirAndCreateRequestResource
            Type of FHIR resource to create. Use 'auto' for automatic resource type detection, or specify a supported US Core profile.

        text : str
            Natural language text to convert to FHIR resource

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        provider : typing.Optional[str]
            FHIR provider ID - must be a valid UUID from existing FHIR providers. also supports provider by name (e.g. medplum)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Lang2FhirAndCreateResponse
            Successfully created FHIR resource

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.tools.create_fhir_resource(
                phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
                phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
                resource="auto",
                text="Patient John Doe has severe asthma with acute exacerbation",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_fhir_resource(
            resource=resource,
            text=text,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            provider=provider,
            request_options=request_options,
        )
        return _response.data

    async def create_fhir_resources_multi(
        self,
        *,
        text: str,
        provider: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Lang2FhirAndCreateMultiResponse:
        """
        Extracts multiple FHIR resources from natural language text and stores them in a FHIR server.
        Automatically detects Patient, Condition, MedicationRequest, Observation, and other resource types.
        Resources are linked with proper references and submitted as a transaction bundle.
        For FHIR servers that don't auto-resolve urn:uuid references, this endpoint will automatically
        resolve them via PUT requests after the initial bundle creation.

        Parameters
        ----------
        text : str
            Natural language text containing multiple clinical concepts to extract

        provider : str
            FHIR provider ID or name

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        version : typing.Optional[str]
            FHIR version to use

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Lang2FhirAndCreateMultiResponse
            Successfully created FHIR resources

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.tools.create_fhir_resources_multi(
                phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
                phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
                text="John Smith, 45-year-old male, diagnosed with Type 2 Diabetes. Prescribed Metformin 500mg twice daily.",
                provider="medplum",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_fhir_resources_multi(
            text=text,
            provider=provider,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            version=version,
            request_options=request_options,
        )
        return _response.data

    async def search_fhir_resources(
        self,
        *,
        text: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        patient_id: typing.Optional[str] = OMIT,
        practitioner_id: typing.Optional[str] = OMIT,
        count: typing.Optional[int] = OMIT,
        provider: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Lang2FhirAndSearchResponse:
        """
        Converts natural language to FHIR search parameters and executes search in FHIR server

        Parameters
        ----------
        text : str
            Natural language text to convert to FHIR search parameters

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        patient_id : typing.Optional[str]
            Patient ID to filter results

        practitioner_id : typing.Optional[str]
            Practitioner ID to filter results

        count : typing.Optional[int]
            Maximum number of results to return

        provider : typing.Optional[str]
            FHIR provider ID - must be a valid UUID from existing FHIR providers. also supports provider by name (e.g. medplum)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Lang2FhirAndSearchResponse
            Successfully generated search and retrieved results

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.tools.search_fhir_resources(
                phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
                phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
                text="Find all appointments for patient John Doe next week",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.search_fhir_resources(
            text=text,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            patient_id=patient_id,
            practitioner_id=practitioner_id,
            count=count,
            provider=provider,
            request_options=request_options,
        )
        return _response.data

    async def analyze_cohort(
        self,
        *,
        text: str,
        provider: str,
        phenoml_on_behalf_of: typing.Optional[str] = None,
        phenoml_fhir_provider: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CohortResponse:
        """
        Uses LLM to extract search concepts from natural language and builds patient cohorts with inclusion/exclusion criteria

        Parameters
        ----------
        text : str
            Natural language text describing the patient cohort criteria

        provider : str
            FHIR provider ID - must be a valid UUID from existing FHIR providers. also supports provider by name (e.g. medplum)

        phenoml_on_behalf_of : typing.Optional[str]
            Optional header for on-behalf-of authentication. Used when making requests on behalf of another user or entity.
            Must be in the format: Patient/{uuid} or Practitioner/{uuid}

        phenoml_fhir_provider : typing.Optional[str]
            Optional header for FHIR provider authentication. Contains credentials in the format {fhir_provider_id}:{oauth2_token}.
            Multiple FHIR provider integrations can be provided as comma-separated values.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CohortResponse
            Successfully analyzed cohort and retrieved patient list

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.tools.analyze_cohort(
                phenoml_on_behalf_of="Patient/550e8400-e29b-41d4-a716-446655440000",
                phenoml_fhir_provider="550e8400-e29b-41d4-a716-446655440000:eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c...",
                text="female patients over 20 with diabetes but not hypertension",
                provider="550e8400-e29b-41d4-a716-446655440000",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.analyze_cohort(
            text=text,
            provider=provider,
            phenoml_on_behalf_of=phenoml_on_behalf_of,
            phenoml_fhir_provider=phenoml_fhir_provider,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

FhirPatchRequestBodyItemOp = typing.Union[
    typing.Literal["add", "remove", "replace", "move", "copy", "test"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .workflow_step_summary_type import WorkflowStepSummaryType


class WorkflowStepSummary(UniversalBaseModel):
    """
    Simplified workflow step without implementation details
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the step
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Human-readable name for the step
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Detailed description of what the step does
    """

    type: typing.Optional[WorkflowStepSummaryType] = pydantic.Field(default=None)
    """
    Type of operation this step performs
    """

    provider_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Selected FHIR provider ID for this step
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SmartConfiguration(UniversalBaseModel):
    """
    SMART on FHIR configuration for OAuth-based providers
    """

    authorization_endpoint: typing.Optional[str] = pydantic.Field(default=None)
    """
    OAuth2 authorization endpoint
    """

    token_endpoint: typing.Optional[str] = pydantic.Field(default=None)
    """
    OAuth2 token endpoint
    """

    issuer: typing.Optional[str] = pydantic.Field(default=None)
    """
    OIDC issuer URL
    """

    jwks_uri: typing.Optional[str] = pydantic.Field(default=None)
    """
    JSON Web Key Set URI
    """

    scopes_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of supported scopes
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class Citation(UniversalBaseModel):
    """
    A reference to source text that led to a code extraction
    """

    text: str = pydantic.Field()
    """
    The exact text span containing evidence for the code
    """

    begin_offset: int = pydantic.Field()
    """
    Starting byte offset in the original input text (0-indexed)
    """

    end_offset: int = pydantic.Field()
    """
    Ending byte offset (exclusive), such that input[begin_offset:end_offset] == text
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .workflow_config import WorkflowConfig
from .workflow_response_graph import WorkflowResponseGraph


class WorkflowResponse(UniversalBaseModel):
    """
    Simplified workflow representation without implementation details
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the workflow
    """

    user_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the user who created the workflow
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Human-readable name for the workflow
    """

    workflow_instructions: typing.Optional[str] = pydantic.Field(default=None)
    """
    Natural language instructions that define the workflow logic
    """

    sample_data: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Sample data used for workflow graph generation
    """

    config: typing.Optional[WorkflowConfig] = None
    graph: typing.Optional[WorkflowResponseGraph] = None
    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Timestamp when the workflow was created
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Timestamp when the workflow was last updated
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawSummaryClient, RawSummaryClient
from .types.create_summary_request_fhir_resources import CreateSummaryRequestFhirResources
from .types.create_summary_request_mode import CreateSummaryRequestMode
from .types.create_summary_response import CreateSummaryResponse
from .types.create_summary_template_response import CreateSummaryTemplateResponse
from .types.summary_delete_template_response import SummaryDeleteTemplateResponse
from .types.summary_get_template_response import SummaryGetTemplateResponse
from .types.summary_list_templates_response import SummaryListTemplatesResponse
from .types.summary_update_template_response import SummaryUpdateTemplateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SummaryClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSummaryClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSummaryClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSummaryClient
        """
        return self._raw_client

    def list_templates(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SummaryListTemplatesResponse:
        """
        Retrieves all summary templates for the authenticated user

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SummaryListTemplatesResponse
            Templates retrieved successfully

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.summary.list_templates()
        """
        _response = self._raw_client.list_templates(request_options=request_options)
        return _response.data

    def create_template(
        self,
        *,
        name: str,
        example_summary: str,
        target_resources: typing.Sequence[str],
        mode: str,
        description: typing.Optional[str] = OMIT,
        example_fhir_data: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateSummaryTemplateResponse:
        """
        Creates a summary template from an example using LLM function calling

        Parameters
        ----------
        name : str
            Name of the template

        example_summary : str
            Example summary note to generate template from

        target_resources : typing.Sequence[str]
            List of target FHIR resources

        mode : str
            Template mode (stored with the template)

        description : typing.Optional[str]
            Description of the template

        example_fhir_data : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Optional example FHIR data that corresponds to the example summary

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateSummaryTemplateResponse
            Template created successfully

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.summary.create_template(
            name="name",
            example_summary="Patient John Doe, age 45, presents with hypertension diagnosed on 2024-01-15.",
            target_resources=["Patient", "Condition", "Observation"],
            mode="mode",
        )
        """
        _response = self._raw_client.create_template(
            name=name,
            example_summary=example_summary,
            target_resources=target_resources,
            mode=mode,
            description=description,
            example_fhir_data=example_fhir_data,
            request_options=request_options,
        )
        return _response.data

    def get_template(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SummaryGetTemplateResponse:
        """
        Retrieves a specific summary template

        Parameters
        ----------
        id : str
            Template ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SummaryGetTemplateResponse
            Template retrieved successfully

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.summary.get_template(
            id="id",
        )
        """
        _response = self._raw_client.get_template(id, request_options=request_options)
        return _response.data

    def update_template(
        self,
        id: str,
        *,
        name: str,
        template: str,
        target_resources: typing.Sequence[str],
        mode: str,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SummaryUpdateTemplateResponse:
        """
        Updates an existing summary template

        Parameters
        ----------
        id : str
            Template ID

        name : str

        template : str
            Updated template with placeholders

        target_resources : typing.Sequence[str]

        mode : str
            Template mode

        description : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SummaryUpdateTemplateResponse
            Template updated successfully

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.summary.update_template(
            id="id",
            name="name",
            template="template",
            target_resources=["target_resources"],
            mode="mode",
        )
        """
        _response = self._raw_client.update_template(
            id,
            name=name,
            template=template,
            target_resources=target_resources,
            mode=mode,
            description=description,
            request_options=request_options,
        )
        return _response.data

    def delete_template(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SummaryDeleteTemplateResponse:
        """
        Deletes a summary template

        Parameters
        ----------
        id : str
            Template ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SummaryDeleteTemplateResponse
            Template deleted successfully

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.summary.delete_template(
            id="id",
        )
        """
        _response = self._raw_client.delete_template(id, request_options=request_options)
        return _response.data

    def create(
        self,
        *,
        fhir_resources: CreateSummaryRequestFhirResources,
        mode: typing.Optional[CreateSummaryRequestMode] = OMIT,
        template_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateSummaryResponse:
        """
        Creates a summary from FHIR resources using one of three modes:
        - **narrative**: Uses a template to substitute FHIR data into placeholders (requires template_id)
        - **flatten**: Flattens FHIR resources into a searchable format for RAG/search (no template needed)
        - **ips**: Generates an International Patient Summary (IPS) narrative per ISO 27269/HL7 FHIR IPS IG. Requires a Bundle with exactly one Patient resource (returns 400 error if no Patient or multiple Patients are present). Automatically filters resources to those referencing the patient and generates sections for allergies, medications, problems, immunizations, procedures, and vital signs.

        Parameters
        ----------
        fhir_resources : CreateSummaryRequestFhirResources
            FHIR resources (single resource or Bundle).
            For IPS mode, must be a Bundle containing exactly one Patient resource with at least one
            identifier (id, fullUrl, or identifier field). Returns an error if no Patient is found,
            if multiple Patients are present, or if the Patient has no identifiers. Resources are
            automatically filtered to only include those referencing the patient.

        mode : typing.Optional[CreateSummaryRequestMode]
            Summary generation mode:
            - narrative: Substitute FHIR data into a template (requires template_id)
            - flatten: Flatten FHIR resources for RAG/search (no template needed)
            - ips: Generate International Patient Summary (IPS) narrative per ISO 27269/HL7 FHIR IPS IG

        template_id : typing.Optional[str]
            ID of the template to use (required for narrative mode)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateSummaryResponse
            Summary generated successfully

        Examples
        --------
        from phenoml import phenoml
        from phenoml.summary import FhirResource

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.summary.create(
            fhir_resources=FhirResource(
                resource_type="resourceType",
            ),
        )
        """
        _response = self._raw_client.create(
            fhir_resources=fhir_resources, mode=mode, template_id=template_id, request_options=request_options
        )
        return _response.data


class AsyncSummaryClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSummaryClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSummaryClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSummaryClient
        """
        return self._raw_client

    async def list_templates(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SummaryListTemplatesResponse:
        """
        Retrieves all summary templates for the authenticated user

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SummaryListTemplatesResponse
            Templates retrieved successfully

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.summary.list_templates()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_templates(request_options=request_options)
        return _response.data

    async def create_template(
        self,
        *,
        name: str,
        example_summary: str,
        target_resources: typing.Sequence[str],
        mode: str,
        description: typing.Optional[str] = OMIT,
        example_fhir_data: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateSummaryTemplateResponse:
        """
        Creates a summary template from an example using LLM function calling

        Parameters
        ----------
        name : str
            Name of the template

        example_summary : str
            Example summary note to generate template from

        target_resources : typing.Sequence[str]
            List of target FHIR resources

        mode : str
            Template mode (stored with the template)

        description : typing.Optional[str]
            Description of the template

        example_fhir_data : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Optional example FHIR data that corresponds to the example summary

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateSummaryTemplateResponse
            Template created successfully

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.summary.create_template(
                name="name",
                example_summary="Patient John Doe, age 45, presents with hypertension diagnosed on 2024-01-15.",
                target_resources=["Patient", "Condition", "Observation"],
                mode="mode",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_template(
            name=name,
            example_summary=example_summary,
            target_resources=target_resources,
            mode=mode,
            description=description,
            example_fhir_data=example_fhir_data,
            request_options=request_options,
        )
        return _response.data

    async def get_template(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SummaryGetTemplateResponse:
        """
        Retrieves a specific summary template

        Parameters
        ----------
        id : str
            Template ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SummaryGetTemplateResponse
            Template retrieved successfully

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.summary.get_template(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_template(id, request_options=request_options)
        return _response.data

    async def update_template(
        self,
        id: str,
        *,
        name: str,
        template: str,
        target_resources: typing.Sequence[str],
        mode: str,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SummaryUpdateTemplateResponse:
        """
        Updates an existing summary template

        Parameters
        ----------
        id : str
            Template ID

        name : str

        template : str
            Updated template with placeholders

        target_resources : typing.Sequence[str]

        mode : str
            Template mode

        description : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SummaryUpdateTemplateResponse
            Template updated successfully

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.summary.update_template(
                id="id",
                name="name",
                template="template",
                target_resources=["target_resources"],
                mode="mode",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_template(
            id,
            name=name,
            template=template,
            target_resources=target_resources,
            mode=mode,
            description=description,
            request_options=request_options,
        )
        return _response.data

    async def delete_template(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SummaryDeleteTemplateResponse:
        """
        Deletes a summary template

        Parameters
        ----------
        id : str
            Template ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SummaryDeleteTemplateResponse
            Template deleted successfully

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.summary.delete_template(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_template(id, request_options=request_options)
        return _response.data

    async def create(
        self,
        *,
        fhir_resources: CreateSummaryRequestFhirResources,
        mode: typing.Optional[CreateSummaryRequestMode] = OMIT,
        template_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateSummaryResponse:
        """
        Creates a summary from FHIR resources using one of three modes:
        - **narrative**: Uses a template to substitute FHIR data into placeholders (requires template_id)
        - **flatten**: Flattens FHIR resources into a searchable format for RAG/search (no template needed)
        - **ips**: Generates an International Patient Summary (IPS) narrative per ISO 27269/HL7 FHIR IPS IG. Requires a Bundle with exactly one Patient resource (returns 400 error if no Patient or multiple Patients are present). Automatically filters resources to those referencing the patient and generates sections for allergies, medications, problems, immunizations, procedures, and vital signs.

        Parameters
        ----------
        fhir_resources : CreateSummaryRequestFhirResources
            FHIR resources (single resource or Bundle).
            For IPS mode, must be a Bundle containing exactly one Patient resource with at least one
            identifier (id, fullUrl, or identifier field). Returns an error if no Patient is found,
            if multiple Patients are present, or if the Patient has no identifiers. Resources are
            automatically filtered to only include those referencing the patient.

        mode : typing.Optional[CreateSummaryRequestMode]
            Summary generation mode:
            - narrative: Substitute FHIR data into a template (requires template_id)
            - flatten: Flatten FHIR resources for RAG/search (no template needed)
            - ips: Generate International Patient Summary (IPS) narrative per ISO 27269/HL7 FHIR IPS IG

        template_id : typing.Optional[str]
            ID of the template to use (required for narrative mode)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateSummaryResponse
            Summary generated successfully

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml
        from phenoml.summary import FhirResource

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.summary.create(
                fhir_resources=FhirResource(
                    resource_type="resourceType",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            fhir_resources=fhir_resources, mode=mode, template_id=template_id, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .execute_workflow_response_results import ExecuteWorkflowResponseResults


class ExecuteWorkflowResponse(UniversalBaseModel):
    success: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the workflow execution was successful
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Status message with execution details
    """

    results: typing.Optional[ExecuteWorkflowResponseResults] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .workflow_definition import WorkflowDefinition
from .workflow_response import WorkflowResponse


class WorkflowsUpdateResponse(UniversalBaseModel):
    success: typing.Optional[bool] = None
    message: typing.Optional[str] = None
    workflow: typing.Optional[WorkflowResponse] = pydantic.Field(default=None)
    """
    Simplified workflow response with only essential step information
    """

    workflow_details: typing.Optional[WorkflowDefinition] = pydantic.Field(default=None)
    """
    Only included when verbose=true - contains full implementation details
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

Lang2FhirAndCreateRequestResource = typing.Union[
    typing.Literal[
        "auto",
        "appointment",
        "condition-encounter-diagnosis",
        "medicationrequest",
        "careplan",
        "condition-problems-health-concerns",
        "coverage",
        "encounter",
        "observation-clinical-result",
        "observation-lab",
        "patient",
        "procedure",
        "questionnaire",
        "questionnaireresponse",
        "simple-observation",
        "vital-signs",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .create_multi_response import CreateMultiResponse
from .create_multi_response_bundle import CreateMultiResponseBundle
from .create_multi_response_bundle_entry_item import CreateMultiResponseBundleEntryItem
from .create_multi_response_bundle_entry_item_request import CreateMultiResponseBundleEntryItemRequest
from .create_multi_response_resources_item import CreateMultiResponseResourcesItem
from .create_request_resource import CreateRequestResource
from .document_request_file_type import DocumentRequestFileType
from .document_request_resource import DocumentRequestResource
from .fhir_resource import FhirResource
from .lang2fhir_upload_profile_response import Lang2FhirUploadProfileResponse
from .search_response import SearchResponse
from .search_response_resource_type import SearchResponseResourceType

__all__ = [
    "CreateMultiResponse",
    "CreateMultiResponseBundle",
    "CreateMultiResponseBundleEntryItem",
    "CreateMultiResponseBundleEntryItemRequest",
    "CreateMultiResponseResourcesItem",
    "CreateRequestResource",
    "DocumentRequestFileType",
    "DocumentRequestResource",
    "FhirResource",
    "Lang2FhirUploadProfileResponse",
    "SearchResponse",
    "SearchResponseResourceType",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from .errors.bad_request_error import BadRequestError
from .errors.conflict_error import ConflictError
from .errors.failed_dependency_error import FailedDependencyError
from .errors.internal_server_error import InternalServerError
from .errors.not_found_error import NotFoundError
from .errors.not_implemented_error import NotImplementedError
from .errors.service_unavailable_error import ServiceUnavailableError
from .errors.unauthorized_error import UnauthorizedError
from .types.construe_upload_code_system_response import ConstrueUploadCodeSystemResponse
from .types.extract_codes_result import ExtractCodesResult
from .types.extract_request_config import ExtractRequestConfig
from .types.extract_request_system import ExtractRequestSystem
from .types.get_code_response import GetCodeResponse
from .types.list_code_systems_response import ListCodeSystemsResponse
from .types.list_codes_response import ListCodesResponse
from .types.semantic_search_response import SemanticSearchResponse
from .types.text_search_response import TextSearchResponse
from .types.upload_request_format import UploadRequestFormat

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawConstrueClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def upload_code_system(
        self,
        *,
        name: str,
        version: str,
        format: UploadRequestFormat,
        file: str,
        revision: typing.Optional[float] = OMIT,
        code_col: typing.Optional[str] = OMIT,
        desc_col: typing.Optional[str] = OMIT,
        defn_col: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ConstrueUploadCodeSystemResponse]:
        """
        Upload a custom medical code system with codes and descriptions for use in code extraction. Requires a paid plan.
        Upon upload, construe generates embeddings for all of the codes in the code system and stores them in the vector database so you can
        subsequently use the code system for construe/extract and lang2fhir/create (coming soon!)

        Parameters
        ----------
        name : str
            Name of the code system

        version : str
            Version of the code system

        format : UploadRequestFormat
            Format of the uploaded file

        file : str
            The file contents as a base64-encoded string

        revision : typing.Optional[float]
            Optional revision number

        code_col : typing.Optional[str]
            Column name containing codes (required for CSV format)

        desc_col : typing.Optional[str]
            Column name containing descriptions (required for CSV format)

        defn_col : typing.Optional[str]
            Optional column name containing long definitions (for CSV format)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ConstrueUploadCodeSystemResponse]
            Successfully uploaded code system
        """
        _response = self._client_wrapper.httpx_client.request(
            "construe/upload",
            method="POST",
            json={
                "name": name,
                "version": version,
                "revision": revision,
                "format": format,
                "file": file,
                "code_col": code_col,
                "desc_col": desc_col,
                "defn_col": defn_col,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConstrueUploadCodeSystemResponse,
                    parse_obj_as(
                        type_=ConstrueUploadCodeSystemResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def extract_codes(
        self,
        *,
        text: str,
        system: typing.Optional[ExtractRequestSystem] = OMIT,
        config: typing.Optional[ExtractRequestConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ExtractCodesResult]:
        """
        Converts natural language text into structured medical codes.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        text : str
            Natural language text to extract codes from

        system : typing.Optional[ExtractRequestSystem]

        config : typing.Optional[ExtractRequestConfig]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ExtractCodesResult]
            Successfully extracted codes
        """
        _response = self._client_wrapper.httpx_client.request(
            "construe/extract",
            method="POST",
            json={
                "text": text,
                "system": convert_and_respect_annotation_metadata(
                    object_=system, annotation=ExtractRequestSystem, direction="write"
                ),
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=ExtractRequestConfig, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractCodesResult,
                    parse_obj_as(
                        type_=ExtractCodesResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_available_code_systems(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ListCodeSystemsResponse]:
        """
        Returns the terminology server's catalog of available code systems, including both built-in standard terminologies and custom uploaded systems.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListCodeSystemsResponse]
            List of available code systems
        """
        _response = self._client_wrapper.httpx_client.request(
            "construe/codes/systems",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListCodeSystemsResponse,
                    parse_obj_as(
                        type_=ListCodeSystemsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_codes_in_a_code_system(
        self,
        codesystem: str,
        *,
        version: typing.Optional[str] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ListCodesResponse]:
        """
        Returns a paginated list of all codes in the specified code system from the terminology server.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        codesystem : str
            Code system name (e.g., "ICD-10-CM", "SNOMED_CT_US_LITE")

        version : typing.Optional[str]
            Specific version of the code system. Required if multiple versions exist.

        cursor : typing.Optional[str]
            Pagination cursor from previous response

        limit : typing.Optional[int]
            Maximum number of codes to return (default 20)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListCodesResponse]
            Paginated list of codes
        """
        _response = self._client_wrapper.httpx_client.request(
            f"construe/codes/{jsonable_encoder(codesystem)}",
            method="GET",
            params={
                "version": version,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListCodesResponse,
                    parse_obj_as(
                        type_=ListCodesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_a_specific_code(
        self,
        codesystem: str,
        code_id: str,
        *,
        version: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetCodeResponse]:
        """
        Looks up a specific code in the terminology server and returns its details.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        codesystem : str
            Code system name

        code_id : str
            The code identifier

        version : typing.Optional[str]
            Specific version of the code system

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetCodeResponse]
            Code details
        """
        _response = self._client_wrapper.httpx_client.request(
            f"construe/codes/{jsonable_encoder(codesystem)}/{jsonable_encoder(code_id)}",
            method="GET",
            params={
                "version": version,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetCodeResponse,
                    parse_obj_as(
                        type_=GetCodeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def semantic_search_embedding_based(
        self,
        codesystem: str,
        *,
        text: str,
        version: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SemanticSearchResponse]:
        """
        Performs semantic similarity search using vector embeddings.

        **When to use**: Best for natural language queries where you want to find conceptually
        related codes, even when different terminology is used. The search understands meaning,
        not just keywords.

        **Examples**:
        - Query "trouble breathing at night" finds codes like "Sleep apnea", "Orthopnea",
          "Nocturnal dyspnea" — semantically related but no exact keyword matches
        - Query "heart problems" finds "Myocardial infarction", "Cardiac arrest", "Arrhythmia"

        **Trade-offs**: Slower than text search (requires embedding generation), but finds
        conceptually similar results that keyword search would miss.

        See also: `/search/text` for faster keyword-based lookup with typo tolerance.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        codesystem : str
            Code system name

        text : str
            Natural language text to find semantically similar codes for

        version : typing.Optional[str]
            Specific version of the code system

        limit : typing.Optional[int]
            Maximum number of results (default 10, max 50)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SemanticSearchResponse]
            Semantic search results ordered by similarity
        """
        _response = self._client_wrapper.httpx_client.request(
            f"construe/codes/{jsonable_encoder(codesystem)}/search/semantic",
            method="GET",
            params={
                "text": text,
                "version": version,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SemanticSearchResponse,
                    parse_obj_as(
                        type_=SemanticSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def terminology_server_text_search(
        self,
        codesystem: str,
        *,
        q: str,
        version: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TextSearchResponse]:
        """
        Performs fast full-text search over code IDs and descriptions.

        **When to use**: Best for autocomplete UIs, code lookup, or when users know part of
        the code ID or specific keywords. Fast response times suitable for typeahead interfaces.

        **Features**:
        - Substring matching on code IDs (e.g., "11.65" finds "E11.65")
        - Typo tolerance on descriptions (not on code IDs)
        - Fast response times (~10-50ms)

        **Examples**:
        - Query "E11" finds all codes starting with E11 (diabetes codes)
        - Query "diabtes" (typo) still finds "diabetes" codes

        **Trade-offs**: Faster than semantic search, but only matches keywords/substrings.
        Won't find conceptually related codes with different terminology.

        See also: `/search/semantic` for finding conceptually similar codes.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        codesystem : str
            Code system name

        q : str
            Search query (searches code IDs and descriptions)

        version : typing.Optional[str]
            Specific version of the code system

        limit : typing.Optional[int]
            Maximum number of results (default 20, max 100)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TextSearchResponse]
            Text search results
        """
        _response = self._client_wrapper.httpx_client.request(
            f"construe/codes/{jsonable_encoder(codesystem)}/search/text",
            method="GET",
            params={
                "q": q,
                "version": version,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TextSearchResponse,
                    parse_obj_as(
                        type_=TextSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawConstrueClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def upload_code_system(
        self,
        *,
        name: str,
        version: str,
        format: UploadRequestFormat,
        file: str,
        revision: typing.Optional[float] = OMIT,
        code_col: typing.Optional[str] = OMIT,
        desc_col: typing.Optional[str] = OMIT,
        defn_col: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ConstrueUploadCodeSystemResponse]:
        """
        Upload a custom medical code system with codes and descriptions for use in code extraction. Requires a paid plan.
        Upon upload, construe generates embeddings for all of the codes in the code system and stores them in the vector database so you can
        subsequently use the code system for construe/extract and lang2fhir/create (coming soon!)

        Parameters
        ----------
        name : str
            Name of the code system

        version : str
            Version of the code system

        format : UploadRequestFormat
            Format of the uploaded file

        file : str
            The file contents as a base64-encoded string

        revision : typing.Optional[float]
            Optional revision number

        code_col : typing.Optional[str]
            Column name containing codes (required for CSV format)

        desc_col : typing.Optional[str]
            Column name containing descriptions (required for CSV format)

        defn_col : typing.Optional[str]
            Optional column name containing long definitions (for CSV format)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ConstrueUploadCodeSystemResponse]
            Successfully uploaded code system
        """
        _response = await self._client_wrapper.httpx_client.request(
            "construe/upload",
            method="POST",
            json={
                "name": name,
                "version": version,
                "revision": revision,
                "format": format,
                "file": file,
                "code_col": code_col,
                "desc_col": desc_col,
                "defn_col": defn_col,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConstrueUploadCodeSystemResponse,
                    parse_obj_as(
                        type_=ConstrueUploadCodeSystemResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def extract_codes(
        self,
        *,
        text: str,
        system: typing.Optional[ExtractRequestSystem] = OMIT,
        config: typing.Optional[ExtractRequestConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ExtractCodesResult]:
        """
        Converts natural language text into structured medical codes.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        text : str
            Natural language text to extract codes from

        system : typing.Optional[ExtractRequestSystem]

        config : typing.Optional[ExtractRequestConfig]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ExtractCodesResult]
            Successfully extracted codes
        """
        _response = await self._client_wrapper.httpx_client.request(
            "construe/extract",
            method="POST",
            json={
                "text": text,
                "system": convert_and_respect_annotation_metadata(
                    object_=system, annotation=ExtractRequestSystem, direction="write"
                ),
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=ExtractRequestConfig, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractCodesResult,
                    parse_obj_as(
                        type_=ExtractCodesResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_available_code_systems(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ListCodeSystemsResponse]:
        """
        Returns the terminology server's catalog of available code systems, including both built-in standard terminologies and custom uploaded systems.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListCodeSystemsResponse]
            List of available code systems
        """
        _response = await self._client_wrapper.httpx_client.request(
            "construe/codes/systems",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListCodeSystemsResponse,
                    parse_obj_as(
                        type_=ListCodeSystemsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_codes_in_a_code_system(
        self,
        codesystem: str,
        *,
        version: typing.Optional[str] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ListCodesResponse]:
        """
        Returns a paginated list of all codes in the specified code system from the terminology server.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        codesystem : str
            Code system name (e.g., "ICD-10-CM", "SNOMED_CT_US_LITE")

        version : typing.Optional[str]
            Specific version of the code system. Required if multiple versions exist.

        cursor : typing.Optional[str]
            Pagination cursor from previous response

        limit : typing.Optional[int]
            Maximum number of codes to return (default 20)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListCodesResponse]
            Paginated list of codes
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"construe/codes/{jsonable_encoder(codesystem)}",
            method="GET",
            params={
                "version": version,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListCodesResponse,
                    parse_obj_as(
                        type_=ListCodesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_a_specific_code(
        self,
        codesystem: str,
        code_id: str,
        *,
        version: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetCodeResponse]:
        """
        Looks up a specific code in the terminology server and returns its details.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        codesystem : str
            Code system name

        code_id : str
            The code identifier

        version : typing.Optional[str]
            Specific version of the code system

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetCodeResponse]
            Code details
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"construe/codes/{jsonable_encoder(codesystem)}/{jsonable_encoder(code_id)}",
            method="GET",
            params={
                "version": version,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetCodeResponse,
                    parse_obj_as(
                        type_=GetCodeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def semantic_search_embedding_based(
        self,
        codesystem: str,
        *,
        text: str,
        version: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SemanticSearchResponse]:
        """
        Performs semantic similarity search using vector embeddings.

        **When to use**: Best for natural language queries where you want to find conceptually
        related codes, even when different terminology is used. The search understands meaning,
        not just keywords.

        **Examples**:
        - Query "trouble breathing at night" finds codes like "Sleep apnea", "Orthopnea",
          "Nocturnal dyspnea" — semantically related but no exact keyword matches
        - Query "heart problems" finds "Myocardial infarction", "Cardiac arrest", "Arrhythmia"

        **Trade-offs**: Slower than text search (requires embedding generation), but finds
        conceptually similar results that keyword search would miss.

        See also: `/search/text` for faster keyword-based lookup with typo tolerance.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        codesystem : str
            Code system name

        text : str
            Natural language text to find semantically similar codes for

        version : typing.Optional[str]
            Specific version of the code system

        limit : typing.Optional[int]
            Maximum number of results (default 10, max 50)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SemanticSearchResponse]
            Semantic search results ordered by similarity
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"construe/codes/{jsonable_encoder(codesystem)}/search/semantic",
            method="GET",
            params={
                "text": text,
                "version": version,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SemanticSearchResponse,
                    parse_obj_as(
                        type_=SemanticSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def terminology_server_text_search(
        self,
        codesystem: str,
        *,
        q: str,
        version: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TextSearchResponse]:
        """
        Performs fast full-text search over code IDs and descriptions.

        **When to use**: Best for autocomplete UIs, code lookup, or when users know part of
        the code ID or specific keywords. Fast response times suitable for typeahead interfaces.

        **Features**:
        - Substring matching on code IDs (e.g., "11.65" finds "E11.65")
        - Typo tolerance on descriptions (not on code IDs)
        - Fast response times (~10-50ms)

        **Examples**:
        - Query "E11" finds all codes starting with E11 (diabetes codes)
        - Query "diabtes" (typo) still finds "diabetes" codes

        **Trade-offs**: Faster than semantic search, but only matches keywords/substrings.
        Won't find conceptually related codes with different terminology.

        See also: `/search/semantic` for finding conceptually similar codes.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        codesystem : str
            Code system name

        q : str
            Search query (searches code IDs and descriptions)

        version : typing.Optional[str]
            Specific version of the code system

        limit : typing.Optional[int]
            Maximum number of results (default 20, max 100)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TextSearchResponse]
            Text search results
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"construe/codes/{jsonable_encoder(codesystem)}/search/text",
            method="GET",
            params={
                "q": q,
                "version": version,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TextSearchResponse,
                    parse_obj_as(
                        type_=TextSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .code_response import CodeResponse
from .code_system_info import CodeSystemInfo


class ListCodesResponse(UniversalBaseModel):
    system: CodeSystemInfo
    codes: typing.List[CodeResponse]
    next_cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    Cursor for fetching the next page (null if no more results)
    """

    has_more: bool = pydantic.Field()
    """
    Whether there are more results available
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from .errors.bad_request_error import BadRequestError
from .errors.forbidden_error import ForbiddenError
from .errors.internal_server_error import InternalServerError
from .errors.not_found_error import NotFoundError
from .errors.unauthorized_error import UnauthorizedError
from .types.create_summary_request_fhir_resources import CreateSummaryRequestFhirResources
from .types.create_summary_request_mode import CreateSummaryRequestMode
from .types.create_summary_response import CreateSummaryResponse
from .types.create_summary_template_response import CreateSummaryTemplateResponse
from .types.summary_delete_template_response import SummaryDeleteTemplateResponse
from .types.summary_get_template_response import SummaryGetTemplateResponse
from .types.summary_list_templates_response import SummaryListTemplatesResponse
from .types.summary_update_template_response import SummaryUpdateTemplateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSummaryClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_templates(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SummaryListTemplatesResponse]:
        """
        Retrieves all summary templates for the authenticated user

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SummaryListTemplatesResponse]
            Templates retrieved successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            "fhir2summary/templates",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SummaryListTemplatesResponse,
                    parse_obj_as(
                        type_=SummaryListTemplatesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_template(
        self,
        *,
        name: str,
        example_summary: str,
        target_resources: typing.Sequence[str],
        mode: str,
        description: typing.Optional[str] = OMIT,
        example_fhir_data: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateSummaryTemplateResponse]:
        """
        Creates a summary template from an example using LLM function calling

        Parameters
        ----------
        name : str
            Name of the template

        example_summary : str
            Example summary note to generate template from

        target_resources : typing.Sequence[str]
            List of target FHIR resources

        mode : str
            Template mode (stored with the template)

        description : typing.Optional[str]
            Description of the template

        example_fhir_data : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Optional example FHIR data that corresponds to the example summary

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateSummaryTemplateResponse]
            Template created successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            "fhir2summary/template",
            method="POST",
            json={
                "name": name,
                "description": description,
                "example_summary": example_summary,
                "target_resources": target_resources,
                "example_fhir_data": example_fhir_data,
                "mode": mode,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateSummaryTemplateResponse,
                    parse_obj_as(
                        type_=CreateSummaryTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_template(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SummaryGetTemplateResponse]:
        """
        Retrieves a specific summary template

        Parameters
        ----------
        id : str
            Template ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SummaryGetTemplateResponse]
            Template retrieved successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"fhir2summary/template/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SummaryGetTemplateResponse,
                    parse_obj_as(
                        type_=SummaryGetTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_template(
        self,
        id: str,
        *,
        name: str,
        template: str,
        target_resources: typing.Sequence[str],
        mode: str,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SummaryUpdateTemplateResponse]:
        """
        Updates an existing summary template

        Parameters
        ----------
        id : str
            Template ID

        name : str

        template : str
            Updated template with placeholders

        target_resources : typing.Sequence[str]

        mode : str
            Template mode

        description : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SummaryUpdateTemplateResponse]
            Template updated successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"fhir2summary/template/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "name": name,
                "description": description,
                "template": template,
                "target_resources": target_resources,
                "mode": mode,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SummaryUpdateTemplateResponse,
                    parse_obj_as(
                        type_=SummaryUpdateTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_template(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SummaryDeleteTemplateResponse]:
        """
        Deletes a summary template

        Parameters
        ----------
        id : str
            Template ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SummaryDeleteTemplateResponse]
            Template deleted successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"fhir2summary/template/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SummaryDeleteTemplateResponse,
                    parse_obj_as(
                        type_=SummaryDeleteTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        fhir_resources: CreateSummaryRequestFhirResources,
        mode: typing.Optional[CreateSummaryRequestMode] = OMIT,
        template_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateSummaryResponse]:
        """
        Creates a summary from FHIR resources using one of three modes:
        - **narrative**: Uses a template to substitute FHIR data into placeholders (requires template_id)
        - **flatten**: Flattens FHIR resources into a searchable format for RAG/search (no template needed)
        - **ips**: Generates an International Patient Summary (IPS) narrative per ISO 27269/HL7 FHIR IPS IG. Requires a Bundle with exactly one Patient resource (returns 400 error if no Patient or multiple Patients are present). Automatically filters resources to those referencing the patient and generates sections for allergies, medications, problems, immunizations, procedures, and vital signs.

        Parameters
        ----------
        fhir_resources : CreateSummaryRequestFhirResources
            FHIR resources (single resource or Bundle).
            For IPS mode, must be a Bundle containing exactly one Patient resource with at least one
            identifier (id, fullUrl, or identifier field). Returns an error if no Patient is found,
            if multiple Patients are present, or if the Patient has no identifiers. Resources are
            automatically filtered to only include those referencing the patient.

        mode : typing.Optional[CreateSummaryRequestMode]
            Summary generation mode:
            - narrative: Substitute FHIR data into a template (requires template_id)
            - flatten: Flatten FHIR resources for RAG/search (no template needed)
            - ips: Generate International Patient Summary (IPS) narrative per ISO 27269/HL7 FHIR IPS IG

        template_id : typing.Optional[str]
            ID of the template to use (required for narrative mode)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateSummaryResponse]
            Summary generated successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            "fhir2summary/create",
            method="POST",
            json={
                "mode": mode,
                "template_id": template_id,
                "fhir_resources": convert_and_respect_annotation_metadata(
                    object_=fhir_resources, annotation=CreateSummaryRequestFhirResources, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateSummaryResponse,
                    parse_obj_as(
                        type_=CreateSummaryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSummaryClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_templates(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SummaryListTemplatesResponse]:
        """
        Retrieves all summary templates for the authenticated user

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SummaryListTemplatesResponse]
            Templates retrieved successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fhir2summary/templates",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SummaryListTemplatesResponse,
                    parse_obj_as(
                        type_=SummaryListTemplatesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_template(
        self,
        *,
        name: str,
        example_summary: str,
        target_resources: typing.Sequence[str],
        mode: str,
        description: typing.Optional[str] = OMIT,
        example_fhir_data: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateSummaryTemplateResponse]:
        """
        Creates a summary template from an example using LLM function calling

        Parameters
        ----------
        name : str
            Name of the template

        example_summary : str
            Example summary note to generate template from

        target_resources : typing.Sequence[str]
            List of target FHIR resources

        mode : str
            Template mode (stored with the template)

        description : typing.Optional[str]
            Description of the template

        example_fhir_data : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Optional example FHIR data that corresponds to the example summary

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateSummaryTemplateResponse]
            Template created successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fhir2summary/template",
            method="POST",
            json={
                "name": name,
                "description": description,
                "example_summary": example_summary,
                "target_resources": target_resources,
                "example_fhir_data": example_fhir_data,
                "mode": mode,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateSummaryTemplateResponse,
                    parse_obj_as(
                        type_=CreateSummaryTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_template(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SummaryGetTemplateResponse]:
        """
        Retrieves a specific summary template

        Parameters
        ----------
        id : str
            Template ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SummaryGetTemplateResponse]
            Template retrieved successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"fhir2summary/template/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SummaryGetTemplateResponse,
                    parse_obj_as(
                        type_=SummaryGetTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_template(
        self,
        id: str,
        *,
        name: str,
        template: str,
        target_resources: typing.Sequence[str],
        mode: str,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SummaryUpdateTemplateResponse]:
        """
        Updates an existing summary template

        Parameters
        ----------
        id : str
            Template ID

        name : str

        template : str
            Updated template with placeholders

        target_resources : typing.Sequence[str]

        mode : str
            Template mode

        description : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SummaryUpdateTemplateResponse]
            Template updated successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"fhir2summary/template/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "name": name,
                "description": description,
                "template": template,
                "target_resources": target_resources,
                "mode": mode,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SummaryUpdateTemplateResponse,
                    parse_obj_as(
                        type_=SummaryUpdateTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_template(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SummaryDeleteTemplateResponse]:
        """
        Deletes a summary template

        Parameters
        ----------
        id : str
            Template ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SummaryDeleteTemplateResponse]
            Template deleted successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"fhir2summary/template/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SummaryDeleteTemplateResponse,
                    parse_obj_as(
                        type_=SummaryDeleteTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        fhir_resources: CreateSummaryRequestFhirResources,
        mode: typing.Optional[CreateSummaryRequestMode] = OMIT,
        template_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateSummaryResponse]:
        """
        Creates a summary from FHIR resources using one of three modes:
        - **narrative**: Uses a template to substitute FHIR data into placeholders (requires template_id)
        - **flatten**: Flattens FHIR resources into a searchable format for RAG/search (no template needed)
        - **ips**: Generates an International Patient Summary (IPS) narrative per ISO 27269/HL7 FHIR IPS IG. Requires a Bundle with exactly one Patient resource (returns 400 error if no Patient or multiple Patients are present). Automatically filters resources to those referencing the patient and generates sections for allergies, medications, problems, immunizations, procedures, and vital signs.

        Parameters
        ----------
        fhir_resources : CreateSummaryRequestFhirResources
            FHIR resources (single resource or Bundle).
            For IPS mode, must be a Bundle containing exactly one Patient resource with at least one
            identifier (id, fullUrl, or identifier field). Returns an error if no Patient is found,
            if multiple Patients are present, or if the Patient has no identifiers. Resources are
            automatically filtered to only include those referencing the patient.

        mode : typing.Optional[CreateSummaryRequestMode]
            Summary generation mode:
            - narrative: Substitute FHIR data into a template (requires template_id)
            - flatten: Flatten FHIR resources for RAG/search (no template needed)
            - ips: Generate International Patient Summary (IPS) narrative per ISO 27269/HL7 FHIR IPS IG

        template_id : typing.Optional[str]
            ID of the template to use (required for narrative mode)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateSummaryResponse]
            Summary generated successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fhir2summary/create",
            method="POST",
            json={
                "mode": mode,
                "template_id": template_id,
                "fhir_resources": convert_and_respect_annotation_metadata(
                    object_=fhir_resources, annotation=CreateSummaryRequestFhirResources, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateSummaryResponse,
                    parse_obj_as(
                        type_=CreateSummaryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
